import unittest


class TestRegexPattern(unittest.TestCase):
    def test_regex_hardcoded_array_1(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = ' (/ llnon, llson, llean, llwen /)'
        array_match = re.match(RegexPattern.hardcoded_array, string)
        self.assertEqual(array_match.group(), string)

    def test_regex_hardcoded_array_2(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = ' (/ (zshift + REAL(ji,wp), ji = 1, jpi_crs*jpj_crs) /)'
        array_match = re.match(RegexPattern.hardcoded_array, string)
        self.assertEqual(array_match.group(), string)

    def test_regex_hardcoded_array_3(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = '(/sd(ju)%ln_tint/)'
        array_match = re.match(RegexPattern.hardcoded_array, string)
        self.assertEqual(array_match.group(), string)

    def test_regex_hardcoded_array_4(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = '(/ sdjf%cltype == \'monthly\' .OR. idy == 0 /)'
        array_match = re.match(RegexPattern.hardcoded_array, string)
        self.assertEqual(array_match.group(), string.strip())

    def test_regex_hardcoded_array_4(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = 'RESHAPE( (/pfield1d%val/), (/1,1,SIZE(pfield1d%val)/) )'
        array_match = re.match(RegexPattern.hardcoded_array, string)
        self.assertEqual(array_match, None)

    def test_regex_array_in_write_statement(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = '(iall_rank(idum) ,all_etime(idum),all_ctime(idum),zall_ratio(idum),idum=1, jpnij)'
        array_match = re.search(RegexPattern.array_in_write_statement, string)
        self.assertEqual(array_match.group(), string)

    def test_regex_real_with_precision_declaration_1(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = 'real(wp)  :: costheta'
        if re.search(RegexPattern.general_real_declaration, string, flags=re.I):
            match = re.search(RegexPattern.real_with_precision_declaration, string, flags=re.I)
        self.assertEqual(match.group(), 'real(wp)  ::')

    def test_regex_real_with_precision_declaration_2(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = 'real(wp), pointer, dimension(:,:,:,:,:), intent(inout) :: real_vec_obj'
        if re.search(RegexPattern.general_real_declaration, string, flags=re.I):
            match = re.search(RegexPattern.real_with_precision_declaration, string, flags=re.I)
        self.assertEqual(match.group(), 'real(wp), pointer, dimension(:,:,:,:,:), intent(inout) ::')

    def test_regex_real_with_precision_declaration_3(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = 'REAL(wp) FUNCTION bdy_segs_surf(phu, phv)'
        if re.search(RegexPattern.general_real_declaration, string, flags=re.I):
            match = re.search(RegexPattern.real_with_precision_declaration, string,flags=re.I)
        self.assertEqual(match.group(), 'REAL(wp) FUNCTION')

    def test_regex_real_with_precision_declaration_4(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = 'REAL(KIND=wp) FUNCTION fspott( pft, pfs, pfp )'
        match = ""
        if re.search(RegexPattern.general_real_declaration, string, flags=re.I):
            match = re.search(RegexPattern.real_with_precision_declaration, string, flags=re.I)
        self.assertEqual(match.group(), 'REAL(KIND=wp) FUNCTION')

    def test_regex_real_with_precision_declaration_5(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = 'type(treal_at_qn3d) :: nodval(3)'
        match = ""
        if re.search(RegexPattern.general_real_declaration, string, flags=re.I):
            match = re.search(RegexPattern.general_real_declaration, string, flags=re.I)
        self.assertEqual(match, "")

    def test_regex_is_call_to_function1(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = '   SUBROUTINE asm_inc_init( Kbb, Kmm, Krhs )'
        self.assertIsNone(re.search(RegexPattern.call_to_function, string, flags=re.I))

    def test_regex_is_call_to_function2(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = '   call asm_inc_init( Kbb, Kmm, Krhs )'
        self.assertIsNone(re.search(RegexPattern.call_to_function, string, flags=re.I))

    def test_regex_is_call_to_function3(self):
        import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
        import re
        string = ' asm_inc_init( Kbb, Kmm, Krhs )'
        self.assertEqual(re.search(RegexPattern.call_to_function, string, flags=re.I).group(), 'asm_inc_init(')

if __name__ == '__main__':
    unittest.main()
