import AutoRPE.UtilsRPE.BasicFunctions as BasicFunction
import AutoRPE.UtilsRPE.CallManager as CallManager
import AutoRPE.UtilsRPE.NatureDeterminer as NatureDeterminer
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
import AutoRPE.UtilsRPE.Error as Error
import re


def split_hardcoded_array(string):
    match = re.search(RegexPattern.hardcoded_array, string)
    if match:
        # Remove the parenthesis
        string = match.group(2)
    else:
        return None
    return CallManager.strip_call_arguments("(" + BasicFunction.remove_outer_brackets(string) + ")")


def split_elements_in_operations(string, procedure=None, vault=None):
    if not NatureDeterminer.has_operations(string):
        return None
    operations = ["+", "-", "*", "/"]
    exponentials_DPE = re.findall(RegexPattern.number_exp_DPE_notation, string, re.I)

    string = string.replace("(/", "BEGIN_VECTOR")
    string = string.replace("/)", "END_VECTOR")

    # Substitute exponential with string
    for index, exponential_DPE in enumerate(exponentials_DPE):
        string = string.replace(exponential_DPE, "EXPONENTIAL_DPE" + str(index))

    string = BasicFunction.remove_brackets(string)

    split_elements = split_elements_by_symbols(string, operations, procedure, vault)

    for index, element in enumerate(split_elements):
        split_elements[index] = element.replace("BEGIN_VECTOR", "(/")
        split_elements[index] = split_elements[index].replace("END_VECTOR", "/)")

    # Substitute back exponentials.
    for index, exponential_DPE in enumerate(exponentials_DPE):
        sub = "EXPONENTIAL_DPE" + str(index)
        for element in split_elements:
            if element.count(sub):
                split_elements[split_elements.index(element)] = element.replace(sub, exponential_DPE)
    if len(split_elements) == 1:
        raise Error.SplitElementError("Not able to split element " + split_elements[0])
    return split_elements


def split_elements_in_comparison(string):
    string = BasicFunction.remove_brackets(string.lower())

    elements_in_comparison = [string]
    for operators in NatureDeterminer.comparison_operators:
        # Loop through all operators, because there can be more than one: a == b  .OR. c == 0
        for element in elements_in_comparison:
            if element.count(operators):
                elements_in_comparison.remove(element)
                elements_in_comparison.extend([e for e in element.split(operators)])
    # Something has been split
    if len(elements_in_comparison) > 1:
        return elements_in_comparison
    else:
        return None


def split_elements_by_symbols(string, symbols, procedure, vault):
    functions = CallManager.calls_to_functions(string, procedure, vault)
    dictionary_of_replacements = {}
    if functions:
        for index, function in enumerate(functions):
            dictionary_of_replacements["REPLACEMENT%i" % index] = function
            string = string.replace(function, "REPLACEMENT%i" % index)

    listed_text = [c for c in string]
    parenthesis_counter = 0
    for index, c in enumerate(listed_text):
        if c == "(":
            parenthesis_counter += 1
        elif c == ")":
            parenthesis_counter -= 1
        elif c in symbols and parenthesis_counter == 0:
            listed_text[index] = "#"
    string = "".join(listed_text)
    arguments = [x.strip() for x in string.split("#") if x.strip()]
    for index, argument in enumerate(arguments):
        for key, function in dictionary_of_replacements.items():
            if key in argument:
                arguments[index] = arguments[index].replace(key, function)

    return arguments


def split_vars_by_allocatable(variables):
    allocatables = [var for var in variables if var.type == "rpe_var" and (var.allocatable or var.is_pointer)]
    non_allocatables = [var for var in variables if var.type == "rpe_var" and not var.allocatable]
    return allocatables, non_allocatables


def split_by_appearance_in_namelist(variables):
    in_namelist = [var for var in variables if var.appears_in_namelist]
    not_in_namelist = [var for var in variables if not var.appears_in_namelist]
    return in_namelist, not_in_namelist


def split_vars_by_main_or_not(variables):
    in_main = [var for var in variables if var.procedure.name == "main"]
    not_in_main = [var for var in variables if var.procedure.name != "main"]

    return in_main, not_in_main


def split_read_arguments(string):
    arguments = []

    # Check for structures like a()%b()
    match_arr_struct = re.findall(RegexPattern.array_with_array_member, string, re.I)
    for struct in match_arr_struct:
        arguments.append(struct)
        string = string.replace(struct, "")
        if not string:
            return arguments

    # Check for structures like a()%b
    match_arr_struct = re.findall(RegexPattern.array_with_member, string, re.I)
    for vec in match_arr_struct:
        arguments.append(vec)
        string = string.replace(vec, "")
        if not string:
            return arguments

    # Check for structures like a%b
    match_struct = re.findall(RegexPattern.array_with_member, string, re.I)
    for struct in match_struct:
        arguments.append(struct)
        string = string.replace(struct, "")
        if not string:
            return arguments

    # Check for vectors like a()
    match_vec = re.findall(RegexPattern.array, string, re.I)
    for vec in match_vec:
        arguments.append(vec)
        string = string.replace(vec, "")
        if not string:
            return arguments

    # Checkall vectors are pruned parenthesis can be deleted
    string = string.replace("(", "")
    string = string.replace(")", "")

    # Check for lists
    if string.count("="):
        # remove index of lists, that are integers
        indexes = re.findall(r"\w+=\d+", string)
        for index in indexes:
            string = string.replace(index, "")
    # The rest is just scalar
    for scalar in string.split(","):
        # add just if is not empty
        if not scalar:
            continue
        arguments.append(scalar)
    return arguments


def split_line_container_and_arguments(string):
    # Given a line containing a call to a write function, it returns the list of the arguments and the
    # remaining of the line without the arguments.

    # Locate start of write call
    loc = re.search(r"(\W|^)(write)\W", string, re.I).start(2)
    # Save the part of the string that goes before the call if any
    pre_text = string[:loc]
    # Keep the rest of the string
    post_text = string[loc:]
    # Temporarily replace write statement with all the parameters
    write_statement = BasicFunction.close_brackets(post_text)
    line = post_text.replace(write_statement, "write()")

    # Find match (?)
    pattern = r"^[^\!]* *write\([^()]*\) (.*)"
    matches = re.search(pattern, line, re.IGNORECASE)
    to_replace = matches.groups(0)[0]

    # Prepare line container
    if to_replace.strip():
        line_container = line.replace(to_replace, "###TOREPLACE###")
    else:
        line_container = line
        return line_container, []

    arguments = to_replace.strip()

    if not arguments:
        return line_container, []

    # Fix array constructors
    if arguments.find("(/") == 0:
        arguments = arguments[2:-2].strip()
        line_container = line_container.replace("###TOREPLACE###", "(/ ###TOREPLACE### /)")

    if BasicFunction.is_all_inside_parenthesis(arguments):
        arguments = BasicFunction.remove_outer_brackets(arguments)
        line_container = line_container.replace("###TOREPLACE###", "( ###TOREPLACE### )")

    arguments = split_write_arguments(arguments)

    line_container = pre_text + line_container.replace("write()", write_statement)
    return line_container, arguments


def split_write_arguments(string):
    # Given a line containing a call to a read function, it returns the list of the arguments
    # pattern = "^[^\!]* +read\([^()]*\) *(.*)"
    # matches = re.search(pattern, string, re.IGNORECASE)
    arguments = string

    scan_quotation = [arguments[:i].count("'") for i, c in enumerate(arguments)]
    scan_quotation2 = [arguments[:i].count("\"") for i, c in enumerate(arguments)]
    scan_op_brackets = [arguments[:i].count("(") for i, c in enumerate(arguments)]
    scan_cl_brackets = [arguments[:i].count(")") for i, c in enumerate(arguments)]
    scan_brackets = [i - j for i, j in zip(scan_op_brackets, scan_cl_brackets)]
    split_text = [c for c in arguments]
    split_text = [
        "#" if c == "," and (scan_quotation[i] % 2 == 0 and scan_quotation2[i] % 2 == 0) and not scan_brackets[i] else c
        for i, c in enumerate(split_text)]
    arguments = "".join(split_text)
    arguments = [a for a in arguments.split("#") if a.strip()]
    return arguments


def split_array_in_write_arguments(string):
    if BasicFunction.is_all_inside_parenthesis(string):
        return CallManager.strip_call_arguments("(" + BasicFunction.remove_outer_brackets(string) + ")")


def split_semicolon(string):
    # Trying to use a rarely used character ({) for the temporal replacement
    if string.count(";"):
        # In the case where there's quotes in the text, we should ignore the exclamations inside
        if not string.count("'") and not string.count("\""):
            return string.split(";")
        else:
            # Looking for
            scan_quotation = [string[:i].count("'") for i, c in enumerate(string)]
            scan_quotation2 = [string[:i].count("\"") for i, c in enumerate(string)]
            splitted_text = [c for c in string]
            splitted_text = ["{" if c == ";" and (scan_quotation[i] % 2 == 1 or scan_quotation2[i] % 2 == 1) else c for
                             i, c in
                             enumerate(splitted_text)]
            string = "".join(splitted_text)
            return string.split(";")

    raise AssertionError("string must contain semi-colon")


## PRIVATE FUNCTION
def is_namelist_line(line):
    return line.lower().strip().find("namelist") == 0
