import AutoRPE.UtilsRPE.Getter as Getter
import AutoRPE.UtilsRPE.BasicFunctions as BasicFunctions
import networkx as nx


def create_nodes(variables, graph):
    for v in variables:
        try:
            graph[var_identifier(v)]
        except KeyError:
            dim = v.dimension if v.dimension is not None else 0
            graph.add_node(var_identifier(v),
                           dimension=dim,
                           routine=str(v.procedure.name),
                           module=str(v.procedure.module.name),
                           intent=str(v.intent),
                           type=str(v.type))


def create_edges(argument, dummy, graph):
    v1, v2 = argument, dummy
    dim = v1.dimension if v1.dimension is not None else 0
    graph.add_edge(var_identifier(v1), var_identifier(v2),
                   routine=str(v2.procedure.name),
                   kind=str(v2.intent),
                   weight=1 + dim / 4.
                   )


def track_dependencies(vault, graph):

    total = len(vault.modules)

    for index, module in enumerate(vault.modules):
        print("\rTracing dependencies %i/%i  %20s" % (index + 1, total, module.name), end="")

        for index, line in enumerate(module.lines):
            line = BasicFunctions.remove_if_condition(line)
            # Returns all REAL var on the line and corresponding dummy argument
            dummy_var, arg = Getter.get_real_in_line(line, vault, module.blocks[index])
            if [dummy_var, arg] != [None, None]:
                # From the argument, get the variable that is bound
                for dv, v in zip(dummy_var, arg):
                    # Create nodes
                    create_nodes([v.variable, dv], graph)
                    # Add edges
                    create_edges(v.variable, dv, graph)


def propagate_dependencies(vault, graph_name):

    # Creates a graph to be filled
    graph = nx.DiGraph()

    # Track dependencies
    track_dependencies(vault, graph)

    nx.readwrite.gexf.write_gexf(graph,  graph_name)
    variables_in_graph = [v for v in vault.variables if var_identifier(v) in graph.nodes]
    var_dict = {}

    for v in variables_in_graph:
        var_dict[var_identifier(v)] = v
    undirected_graph = graph.to_undirected()

    # Stores info about these dependencies into the vault
    for v in variables_in_graph:
        cc = nx.node_connected_component(undirected_graph, var_identifier(v))
        try:
            v.same_as = [var_dict[n] for n in cc]
        except KeyError:
            raise KeyError


def var_identifier(var):
    return "%s#%s#%s#%s" % (var.name, var.procedure.name, var.procedure.module.name, str(var.id))
